package agentes;

import java.util.Random;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;

public class Waiter extends Agent {

	private static final long serialVersionUID = 505503825544386700L;
	private DFAgentDescription dfd;
	private int food[], drinks[];
	private boolean takingSupplies = false;

	public void setup() {

		System.out.println("Agente " + getLocalName() + " ha empezado.");
		dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("waiter");
		sd.setName("waiter-service");
		dfd.addServices(sd);
		food = new int[] { 0, 0, 0 };
		drinks = new int[] { 0, 0, 0 };
		try {
			Thread.sleep(10000); // Dar tiempo a que entre algun invitado
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		addBehaviour(new CheckFoodAndDrinks(this, 4000));
		addBehaviour(new FindCustomers(this, 6000));
		addBehaviour(new WaitForResponse(this));
	}

	private class CheckFoodAndDrinks extends TickerBehaviour {
		private static final long serialVersionUID = -6511328046472603543L;

		public CheckFoodAndDrinks(Agent a, long period) {
			super(a, period);
		}

		@Override

		public void onTick() {
			System.out
					.println("Agente " + getLocalName() + " TickerBehaviour - Compobando cantidad de comida y bebida.");
			// printState();
			int falta = 0;
			for (int i = 0; i <= 2; i++) {
				falta += (food[i] <= 0) ? 1 : 0;
				falta += (drinks[i] <= 0) ? 1 : 0;
			}

			if (falta >= 5 && !takingSupplies) {
				addBehaviour(new OneShotBehaviour() {
					private static final long serialVersionUID = -5103559308618758579L;

					@Override
					public void action() {
						takingSupplies = true;
						System.out.println("Agente " + getLocalName()
								+ " OneShotBehaviour - Necesito mas comida y bebida. Voy a la barra a coger.");

						DFAgentDescription template = new DFAgentDescription();
						ServiceDescription sd = new ServiceDescription();
						sd.setType("barman");
						sd.setName("barman-service");
						template.addServices(sd);
						try {
							DFAgentDescription[] result = DFService.search(myAgent, template);
							if (result.length > 0) {
								AID r = new AID();
								r.setLocalName(result[0].getName().getLocalName());
								ACLMessage msg = new ACLMessage(ACLMessage.PROPOSE);
								msg.setSender(getAID());
								msg.addReceiver(r);
								msg.setContent("FoodAndDrinkRequest");
								System.out.println("Agente " + getLocalName()
										+ "Enviado mensaje QUERY_REF con contenido: Necesito mas comida y bebida.");
								send(msg);
							} else {
								System.out.println("No hay Barman.!!!");
								takingSupplies = false;
							}
						} catch (FIPAException fe) {
							fe.printStackTrace();
						}
					}
				});
			}

			try {
				Thread.sleep(1500);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	private class FindCustomers extends TickerBehaviour {
		private static final long serialVersionUID = 7277977699198704399L;

		public FindCustomers(Agent a, long period) {
			super(a, period);
		}

		@Override
		protected void onTick() {
			if (!takingSupplies) {
				DFAgentDescription template = new DFAgentDescription();
				ServiceDescription sd = new ServiceDescription();
				sd.setType("EnFiesta");
				template.addServices(sd);
				try {
					DFAgentDescription[] result = DFService.search(myAgent, template);
					AID r = new AID();
					Random rand = new Random();
					if (result.length >= 1) {
						int customer = rand.nextInt(result.length);
						r.setLocalName(result[customer].getName().getLocalName());
						ACLMessage msg = new ACLMessage(ACLMessage.QUERY_REF);
						msg.setSender(getAID());
						msg.addReceiver(r);
						msg.setContent("FoodAndDrink");
						System.out.println("Agente " + getLocalName() + " Enviado mensaje QUERY_REF a "
								+ result[customer].getName().getLocalName() + " con contenido: Que desea Tomar.");
						send(msg);
					} else {
						System.out.println("No Hay Invitados en la fiesta.");
						myAgent.doDelete();
					}
				} catch (FIPAException fe) {
					fe.printStackTrace();
				}
			}
		}
	}

	class WaitForResponse extends CyclicBehaviour {
		private static final long serialVersionUID = 7277977699198704399L;

		public WaitForResponse(Agent a) {
			super(a);
		}

		public void action() {
			ACLMessage msg = receive();
			if (msg != null) {
				if (msg.getPerformative() == ACLMessage.NOT_UNDERSTOOD) {
					System.out.println("Mensaje NOT UNDERSTOOD recibido");
				} else {
					if (msg.getPerformative() == ACLMessage.ACCEPT_PROPOSAL) {
						// **************** BARMAN ****************
						resetFoodAndDriks();
						takingSupplies = false;
					} else if (msg.getPerformative() == ACLMessage.INFORM) {
						// **************** INVITADO ****************
						ACLMessage reply = msg.createReply();
						System.out.println("Agente " + getLocalName() + " Enviado mensaje INFORM a "
								+ msg.getSender().getLocalName() + " con contenido: de Acuerdo, aqui tiene.");
						char response[] = new char[2];
						response = msg.getContent().toCharArray();

						if (response[0] == '0') {
							if (response[1] <= '2' && response[1] >= '0')
								food[Integer.parseInt(response[1] + "")]--;
						} else if (response[0] == '1') {
							if (response[1] <= '2' && response[1] >= '0')
								drinks[Integer.parseInt(response[1] + "")]--;
						}
						printState();
						try {
							Thread.sleep(2000);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						reply.setContent("TakeForFoodOrDrink");
						send(reply);
					} else if (msg.getPerformative() == ACLMessage.CANCEL) {
						if (msg.getContent() == "END")
							myAgent.doDelete();
					} else {
						block();
					}
				}
			}
		}
	}

	protected void takeDown() {
		try {
			Thread.sleep(15000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		System.out.println("Agente " + getLocalName()
				+ " Eliminando servicios de Paginas Amarillas, Saliendo de la fiesta tras un largo dia.");
		AID r = new AID();
		DFAgentDescription template = new DFAgentDescription();
		ServiceDescription sd = new ServiceDescription();
		sd.setType("barman");
		sd.setName("barman-service");
		template.addServices(sd);
		DFAgentDescription[] result;
		try {
			result = DFService.search(this, template);

			if (result.length > 0) {
				r = new AID();
				r.setLocalName(result[0].getName().getLocalName());
				ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
				msg.setSender(getAID());
				msg.addReceiver(r);
				msg.setContent("END");
				System.out.println("Agente " + getLocalName()
						+ "Enviado mensaje QUERY_REF con contenido: Necesito mas comida y bebida.");
				send(msg);
			}
		} catch (FIPAException e) {
			e.printStackTrace();
		}
	}

	private void printState() {
		if (!takingSupplies) {
			System.out.println("========= COMIDA =========");
			System.out.println("\t 0:" + food[0] + "\t 2:" + food[1] + "\t 3:" + food[2]);
			System.out.println("========= BEBIDA =========");
			System.out.println("\t 0:" + drinks[0] + "\t 2:" + drinks[1] + "\t 3:" + drinks[2]);
			System.out.println();
		}
	}

	private void resetFoodAndDriks() {
		System.out.println("========= FOOD&DRINK RESET =========");
		for (int i = 0; i <= 2; i++) {
			food[i] = 10;
			drinks[i] = 5;
		}
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}